# Copyright 2013 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

"""Tests for `maastest.utils`."""

from __future__ import (
    absolute_import,
    print_function,
    unicode_literals,
    )

__metaclass__ = type
__all__ = []

import os.path
from subprocess import PIPE

from fixtures import TempDir
from maastest import utils
import mock
import testtools


class TestRunCommand(testtools.TestCase):

    def test_read_file_returns_file_contents_as_bytes(self):
        temp_dir = self.useFixture(TempDir())
        sample_file = os.path.join(temp_dir.path, self.getUniqueString())
        contents = self.getUniqueString().encode('ascii')
        with open(sample_file, 'wb') as f:
            f.write(contents)

        read_contents = utils.read_file(sample_file)

        self.assertEqual(contents, read_contents)
        self.assertIsInstance(contents, bytes)

    def test_run_command_calls_Popen(self):
        mock_Popen = mock.MagicMock()
        expected_retcode = self.getUniqueInteger()
        expected_stdout = self.getUniqueString()
        expected_stderr = self.getUniqueString()
        mock_Popen.return_value.returncode = expected_retcode
        mock_Popen.return_value.communicate.return_value = (
            expected_stdout, expected_stderr)
        self.patch(utils, 'Popen', mock_Popen)
        args = ['one', 'two']

        retcode, stdout, stderr = utils.run_command(args)

        self.assertEqual(
            [
                mock.call(
                    [u'one', u'two'], stdout=PIPE, stderr=PIPE,
                    stdin=PIPE, shell=False),
                mock.call().communicate(None),
            ],
            mock_Popen.mock_calls)
        self.assertEqual(
            (expected_retcode, expected_stdout, expected_stderr),
            (retcode, stdout, stderr))

    def test_run_command_runs_command(self):
        retcode, stdout, stderr = utils.run_command(['ls', '/'])

        self.assertEqual((0, ''), (retcode, stderr))
        self.assertIn("boot", stdout)

    def test_run_command_checks_return_value(self):
        mock_Popen = mock.MagicMock()
        expected_retcode = 2
        expected_stdout = self.getUniqueString()
        expected_stderr = self.getUniqueString()
        mock_Popen.return_value.returncode = expected_retcode
        mock_Popen.return_value.communicate.return_value = (
            expected_stdout, expected_stderr)
        self.patch(utils, 'Popen', mock_Popen)
        args = ['one', 'two']

        error = self.assertRaises(
            Exception, utils.run_command, args, check_call=True)
        self.assertIn(expected_stdout, error.message)
        self.assertIn(expected_stderr, error.message)

    def test_run_command_uses_input(self):
        input_string = self.getUniqueString()
        retcode, stdout, stderr = utils.run_command(
            ['cat', '-'], input=input_string)

        self.assertEqual(
            (0, '', input_string), (retcode, stderr, stdout))
